Module construction_matrice
    use Mod_DONNEES
    implicit none

CONTAINS

    !************maillage********************
    subroutine grille ( dr, dz, r, z)
        implicit none
        integer:: i, j
        REAL*8, intent(out) :: dr, dz
        REAL*8, DIMENSION(:), intent(out) :: r
        REAL*8, DIMENSION(:), intent(out) :: z

        !les pas dans l'espace
        dr = ( r_max - r_min ) / m
        dz = Lt / n

        !calcul de z(i)
        do i = 1, n
            z(i)= (dz/(2.0D0))+((i-1)*dz)
        end do

        !calcul de r(j)
        do j = 1, m
            r(j)= (dr*0.5D0)-(j*dr)+r_max
        end do

    end subroutine grille


    !************remplissage de la matrice*************
    subroutine matrice ( r, z, dr, dz, vect, A)
        implicit none
        REAL*8, DIMENSION(:), intent(in) :: r
        REAL*8, DIMENSION(:), intent(in) :: z
        REAL*8, intent(in) :: dr
        REAL*8, intent(in) :: dz
        REAL*8, DIMENSION(:,:), intent(out) :: vect
        REAL*8, DIMENSION(:,:), intent(out) :: A
        REAL*8 :: L_test
        INTEGER :: l,l1,l2,l3,l4
        INTEGER :: i,j

        L_test = L_evap + L_adiab

        !******Element1**********************
        i=1
        j=1
        l=(j-1)*n+i
        l1=(j-1)*n+i+1
        l3=j*n+i

        A(l,l)  = (r(j)*(dr/dz)) +(r(j)*(dz/dr))-(dz/(2.0D0))
        A(l,l1) = (-1.0D0)*r(j)*(dr/dz)
        A(l,l3) = (-1.0D0)*(r(j)-(dr/2.0D0))*(dz/dr)

        vect(l,1)=q_evap/((2.0D0)*Pi*conds)

        !******Element2************************
        j=1
        do i = 2, n-1
            j=1
            l=(j-1)*n+i
            l1=(j-1)*n+i+1
            l2=(j-1)*n+i-1
            l3=j*n+i

            A(l,l2) = (-1.0D0)*r(j)*(dr/dz)
            A(l,l1) = (-1.0D0)*r(j)*(dr/dz)
            A(l,l3) = (-1.0D0)*(r(j)-(dr/2.0D0))*(dz/dr)

            if (z(i)< L_evap .and. z(i)==L_evap) then
                A(l,l)=((2.0D0)*r(j)*(dr/dz))+(r(j)*(dz/dr))-(dz/2.0D0)
                vect(l,1)=q_evap/((2.0D0)*Pi*conds)

            else if ( z(i) > L_evap .and. z(i) < L_test .and. z(i)==L_test) then
                A(l,l)=((2.0D0)*r(j)*(dr/dz))+(r(j)*(dz/dr))-(dz/2.0D0)
                vect(l,1)= 0.0D0

            else
                A(l,l)=((2.0D0)*r(j)*(dr/dz))+(coef/(2.0D0*Pi*conds))+(r(j)*(dz/dr))-(dz/2.0D0)
                vect(l,1)=((-1.0D0)*(coef*T_inf))/(2.0D0*Pi*conds)
            end if

        end do

        !******Element3**************************
        i=n
        j=1
        l=(j-1)*n+i
        l1=(j-1)*n+i+1
        l2=(j-1)*n+i-1
        l3=j*n+i

        A(l,l)  = (r(j)*(dr/dz))+(r(j)*(dz/dr))-(dz/2.0D0)-(coef/(2.0D0*Pi*conds))
        A(l,l2) = (-1.0D0)*(dr/dz)
        A(l,l3) = (-1.0D0)*(r(j)-(dr/2.0D0))*(dz/dr)
        vect(l,1) = ((-1.0D0)*(coef*T_inf))/(2.0D0*Pi*conds)

        !******Element4***************************
        i=1
        do j=2,m-1
            l=(j-1)*n+i
            l1=(j-1)*n+i+1
            l2=(j-1)*n+i-1
            l3=j*n+i
            l4=(j-2)*n+i

            A(l,l)=(r(j)*(dr/dz))+(2.0D0*r(j)*(dz/dr))
            A(l,l1)=(-1.0D0)*r(j)*(dr/dz)
            A(l,l4)=(-1.0D0)*(r(j)+(dr/2.0D0))*(dz/dr)
            A(l,l3)=(-1.0D0)*(r(j)-(dr/2.0D0))*(dz/dr)

            vect(l,1)=0.0D0
        end do

        !******Element5***************************
        do i=2,n-1
            do j=2,m-1

                l=(j-1)*n+i
                l1=(j-1)*n+i+1
                l2=(j-1)*n+i-1
                l3=j*n+i
                l4=(j-2)*n+i

                A(l,l)=(2.0D0*r(j)*(dr/dz))+(2.0D0*r(j)*(dz/dr))
                A(l,l2)=(-1.0D0)*r(j)*(dr/dz)
                A(l,l1)=(-1.0D0)*r(j)*(dr/dz)
                A(l,l4)=(-1.0D0)*(r(j)+(dr/2.0D0))*(dz/dr)
                A(l,l3)=(-1.0D0)*(r(j)-(dr/2.0D0))*(dz/dr)

                vect(l,1)=0.0D0
            end do
        end do

        !******Element6*****************************
        i=n
        do j=2,m-1

            l=(j-1)*n+i
            l2=(j-1)*n+i-1
            l3=j*n+i
            l4=(j-2)*n+i

            A(l,l)=(r(j)*(dr/dz))+(2.0D0*r(j)*(dz/dr))
            A(l,l2)=(-1.0D0)*r(j)*(dr/dz)
            A(l,l4)=(-1.0D0)*(r(j)+(dr/2.0D0))*(dz/dr)
            A(l,l3)=(-1.0D0)*(r(j)-(dr/2.0D0))*(dz/dr)
            vect(l,1)=0.0D0
        end do

        !******Element7*****************************
        i=1
        j=m
        l=(j-1)*n+i
        l1=(j-1)*n+i+1
        l4=(j-2)*n+i

        A(l,l)=(r(j)*(dr/dz))+(r(j)*(dz/dr))+(dz/2.0D0)
        A(l,l1)=(-1.0D0)*r(j)*(dr/dz)
        A(l,l4)=(-1.0D0)*(r(j)+(dr/2.0D0))*(dz/dr)
        vect(l,1)=0.0D0

        !******Element8*****************************
        j=m
        do i=2,n-1

            l=(j-1)*n+i
            l1=(j-1)*n+i+1
            l2=(j-1)*n+i-1
            l4=(j-2)*n+i

            A(l,l)=(2.0D0*r(j)*(dr/dz))+(r(j)*(dz/dr))+(dz/2.0D0)
            A(l,l2)=(-1.0D0)*r(j)*(dr/dz)
            A(l,l1)=(-1.0D0)*r(j)*(dr/dz)
            A(l,l4)=(-1.0D0)*(r(j)+(dr/2.0D0))*(dz/dr)
            vect(l,1)=0.0D0
        end do

        !******Element9******************************
        i=n
        j=m
        l=(j-1)*n+i
        l2=(j-1)*n+i-1
        l4=(j-2)*n+i

        A(l,l)=(r(j)*(dr/dz))+(r(j)*(dz/dr))+(dz/2.0D0)
        A(l,l2)=(-1.0D0)*r(j)*(dr/dz)
        A(l,l4)=(-1.0D0)*(r(j)+(dr/2.0D0))*(dz/dr)
        vect(l,1)=0.0D0

    end subroutine matrice

END Module construction_matrice
